<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://hashthemes.com/
 * @since      1.0.0
 *
 * @package    Super_Floating_Flying_Menu
 * @subpackage Super_Floating_Flying_Menu/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Super_Floating_Flying_Menu
 * @subpackage Super_Floating_Flying_Menu/admin
 * @author     Hash Themes 
 */
class Super_Floating_Flying_Menu_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Holder for current menu ID
     * @var integer
     */
    protected $current_menu_id = null;

    /**
     * Total Menu settings page
     *
     * @var string
     */
    protected $menu_item_meta_key = 'sffm_menu_settings';

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->load_files();

        add_theme_support('menus');

        add_action('admin_head-nav-menus.php', array($this, 'register_nav_meta_box'), 9);
        add_action('wp_ajax_sffm_save_menu_item_data', array($this, 'save_menu_item_data'));

        /* Metabox select menu as sffm-menu */
        add_action('wp_ajax_sffm_menu_select', array($this, 'sffm_menu_select'));
        add_action('wp_ajax_sffm_menu_deselect', array($this, 'sffm_menu_deselect'));

        /* Floating or Flying Menu */
        add_action('wp_ajax_sffm_menu_floatfly', array($this, 'sffm_menu_floatfly'));
        add_action('wp_ajax_sffm_save_floatfly_menu_data', array($this, 'save_floatfly_menu_data'));

        add_action('admin_enqueue_scripts', array($this, 'enqueue_styles'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));

        add_action('admin_init', array($this, 'update_menu_template'));
        add_action('wp_delete_nav_menu', array($this, 'delete_nav_menu_action'));

        add_filter('plugin_action_links_' . plugin_basename(SFFM_FILE), array($this, 'add_plugin_action_link'), 10, 1);

        add_action('wp_delete_nav_menu', array($this, 'update_sffm_settings'), 10, 1);

        add_action('init', array($this, 'register_translation_strings'));
    }

    public function load_files() {
        require plugin_dir_path(__FILE__) . 'inc/sffm-menu.php';
        require plugin_dir_path(__FILE__) . 'inc/register-elementor-content-cpt.php';
        require plugin_dir_path(__FILE__) . 'inc/helper-function.php';
        require plugin_dir_path(__FILE__) . 'inc/font-icons.php';
        require plugin_dir_path(__FILE__) . 'inc/google-fonts-list.php';
        require plugin_dir_path(__FILE__) . 'inc/elements/elements.php';
        require plugin_dir_path(__FILE__) . 'inc/admin-page.php';
        require plugin_dir_path(__FILE__) . 'inc/admin-notice.php';
        require plugin_dir_path(__FILE__) . 'inc/handle-import-export.php';
    }

    /**
     * Ajax function to save floatfly menu data
     */
    public function save_floatfly_menu_data() {

        if (isset($_POST['wp_nonce']) && wp_verify_nonce($_POST['wp_nonce'], 'sffm-backend-ajax-nonce')) {
            if (isset($_POST['data']) && !empty($_POST['data'])) {
                $menu_id = isset($_POST['menu_id']) ? sanitize_text_field($_POST['menu_id']) : null;
                $menu_type = isset($_POST['menu_type']) ? sanitize_text_field($_POST['menu_type']) : null;

                $menu_type_settings = array();
                $menu = 'menu_' . $menu_id;
                parse_str($_POST['data'], $menu_type_settings);

                $sffm_settings = get_option('sffm_settings');

                /* get the assigned menu type for the menuid */
                $sffm_settings[$menu]['menu_id'] = $menu_id;
                $sffm_settings[$menu]['menu_type'] = $menu_type;

                if ($menu_type == 'floating_menu') {
                    if (isset($menu_type_settings['floatmenu_hide_screen'])) {
                        $sffm_settings[$menu]['floatmenu']['hide_screen'] = $menu_type_settings['floatmenu_hide_screen'];
                    }
                    if (isset($menu_type_settings['floatmenu_display_condition'])) {
                        $sffm_settings[$menu]['floatmenu']['specific_pages'] = isset($menu_type_settings['floatmenu_specific_pages']) ? $menu_type_settings['floatmenu_specific_pages'] : [];
                        $sffm_settings[$menu]['floatmenu']['specific_archive'] = isset($menu_type_settings['floatmenu_specific_archive']) ? $menu_type_settings['floatmenu_specific_archive'] : [];
                        $sffm_settings[$menu]['floatmenu']['cpt_pages'] = isset($menu_type_settings['floatmenu_cpt_pages']) ? $menu_type_settings['floatmenu_cpt_pages'] : [];
                    }
                    if (isset($menu_type_settings['floatmenu_polylang_display_condition'])) {
                        $sffm_settings[$menu]['floatmenu']['polylang_display_condition'] = isset($menu_type_settings['floatmenu_polylang_display_condition']) ? $menu_type_settings['floatmenu_polylang_display_condition'] : [];
                        $sffm_settings[$menu]['floatmenu']['polylang_show_hide_languages'] = isset($menu_type_settings['floatmenu_polylang_show_hide_languages']) ? $menu_type_settings['floatmenu_polylang_show_hide_languages'] : [];
                    }
                    if (isset($menu_type_settings['floatmenu_button_shadow']['x'])) {
                        $sffm_settings[$menu]['floatmenu']['button_shadow']['x'] = is_numeric($menu_type_settings['floatmenu_button_shadow']['x']) ? intval($menu_type_settings['floatmenu_button_shadow']['x']) : '';
                    }
                    if (isset($menu_type_settings['floatmenu_button_shadow']['y'])) {
                        $sffm_settings[$menu]['floatmenu']['button_shadow']['y'] = is_numeric($menu_type_settings['floatmenu_button_shadow']['y']) ? intval($menu_type_settings['floatmenu_button_shadow']['y']) : '';
                    }
                    if (isset($menu_type_settings['floatmenu_button_shadow']['blur'])) {
                        $sffm_settings[$menu]['floatmenu']['button_shadow']['blur'] = is_numeric($menu_type_settings['floatmenu_button_shadow']['blur']) ? intval($menu_type_settings['floatmenu_button_shadow']['blur']) : '';
                    }
                    if (isset($menu_type_settings['floatmenu_button_shadow']['color']))
                        $sffm_settings[$menu]['floatmenu']['button_shadow']['color'] = sffm_sanitize_color($menu_type_settings['floatmenu_button_shadow']['color']);
                    if (isset($menu_type_settings['floatmenu_tooltip_line_height'])) {
                        $sffm_settings[$menu]['floatmenu']['tooltip_line_height'] = is_numeric($menu_type_settings['floatmenu_tooltip_line_height']) ? floatval($menu_type_settings['floatmenu_tooltip_line_height']) : '';
                    }

                    /* Text Fields */
                    $floatmenu_text = array(
                        'template',
                        'icon_type',
                        'hamburger_icon_type',
                        'tab_default_icon',
                        'tab_custom_icon',
                        'tab_default_close_icon',
                        'tab_custom_close_icon',
                        'temp_1_position',
                        'temp_1_orientation',
                        'temp_1_tab_shape',
                        'temp_56_tab_shape',
                        'temp_7_position',
                        'temp_8_position',
                        'temp_9_position',
                        'temp_9_orientation',
                        'temp_10_position',
                        'temp_11_position',
                        'temp_12_position',
                        'temp_13_position',
                        'temp_14_position',
                        'temp_14_animation',
                        'animation',
                        'temp_style',
                        'display_condition',
                        'trigger_btn_hvr_animation',
                        'trigger_btn_idle_animation',
                        'open_item_on',
                        'tooltip_font_family',
                        'tooltip_text_transform',
                        'tooltip_text_decoration',
                        'tooltip_font_style'
                    );
                    foreach ($floatmenu_text as $key) {
                        if (isset($menu_type_settings['floatmenu_' . $key]))
                            $sffm_settings[$menu]['floatmenu'][$key] = sanitize_text_field($menu_type_settings['floatmenu_' . $key]);
                    }

                    /* Hex Color */
                    $floatmenu_colors = array(
                        'button_icon_color',
                        'button_hover_icon_color',
                        'nav_item_bg_color',
                        'nav_item_border_color',
                        'nav_item_hover_border_color',
                        'nav_item_hover_bg_color',
                        'nav_item_active_bg_color',
                        'nav_item_active_border_color',
                        'tooltip_text_color',
                        'tooltip_hover_text_color',
                        'trigger_btn_icon_color',
                        'trigger_btn_hover_icon_color'
                    );
                    foreach ($floatmenu_colors as $key) {
                        if (isset($menu_type_settings['floatmenu_' . $key]))
                            $sffm_settings[$menu]['floatmenu'][$key] = sanitize_hex_color($menu_type_settings['floatmenu_' . $key]);
                    }

                    /* Alpha Color */
                    $floatmenu_alphacolors = array(
                        'button_bg_color',
                        'button_hover_bg_color',
                        'tooltip_bg_color',
                        'tooltip_hover_bg_color',
                        'trigger_btn_bg_color',
                        'trigger_btn_hover_bg_color',
                        'circle_bg_color',
                        'buttons_bg_color',
                    );
                    foreach ($floatmenu_alphacolors as $key) {
                        if (isset($menu_type_settings['floatmenu_' . $key]))
                            $sffm_settings[$menu]['floatmenu'][$key] = sffm_sanitize_color($menu_type_settings['floatmenu_' . $key]);
                    }

                    /* Number */
                    $floatmenu_numbers = array(
                        'temp_1_offset_top',
                        'temp_1_offset_left',
                        'temp_1_offset_bottom',
                        'temp_1_offset_right',
                        'temp_7_offset_top',
                        'temp_7_offset_left',
                        'temp_7_offset_bottom',
                        'temp_7_offset_right',
                        'temp_8_offset_top',
                        'temp_8_offset_left',
                        'temp_8_offset_bottom',
                        'temp_8_offset_right',
                        'temp_9_offset_top',
                        'temp_9_offset_left',
                        'temp_9_offset_bottom',
                        'temp_9_offset_right',
                        'temp_10_offset_top',
                        'temp_10_offset_left',
                        'temp_10_offset_bottom',
                        'temp_10_offset_right',
                        'temp_11_radius',
                        'temp_11_offset_top',
                        'temp_11_offset_left',
                        'temp_11_offset_bottom',
                        'temp_11_offset_right',
                        'temp_12_radius',
                        'temp_12_offset_top',
                        'temp_12_offset_left',
                        'temp_12_offset_bottom',
                        'temp_12_offset_right',
                        'temp_13_radius',
                        'temp_13_offset_top',
                        'temp_13_offset_left',
                        'temp_13_offset_bottom',
                        'temp_13_offset_right',
                        'temp_14_offset_left',
                        'temp_14_offset_right',
                        'temp_14_scroll_top_offset',
                        'button_size',
                        'zindex',
                        'icon_size',
                        'tab_spacing',
                        'nav_item_spacing',
                        'nav_item_size',
                        'trigger_btn_size',
                        'trigger_btn_icon_size',
                        'hamburger_spacing',
                        'hamburger_width',
                        'hamburger_thickness',
                        'tooltip_font_size',
                        'tooltip_letter_spacing'
                    );
                    foreach ($floatmenu_numbers as $key) {
                        if (isset($menu_type_settings['floatmenu_' . $key]))
                            $sffm_settings[$menu]['floatmenu'][$key] = is_numeric($menu_type_settings['floatmenu_' . $key]) ? intval($menu_type_settings['floatmenu_' . $key]) : '';
                    }

                    /* CheckBox */
                    $floatmenu_checkbox = array(
                        'front_pages',
                        'blog_pages',
                        'archive_pages',
                        'error_pages',
                        'search_pages',
                        'open_items_on_load',
                    );

                    foreach ($floatmenu_checkbox as $key) {
                        if (isset($menu_type_settings['floatmenu_' . $key]))
                            $sffm_settings[$menu]['floatmenu'][$key] = $menu_type_settings['floatmenu_' . $key] == 'on' ? 'on' : 'off';
                    }
                } elseif ($menu_type == 'fly_menu') {
                    if (isset($menu_type_settings['flymenu_header_custom_content']))
                        $sffm_settings[$menu]['flymenu']['header_custom_content'] = wp_kses_post($menu_type_settings['flymenu_header_custom_content']);
                    if (isset($menu_type_settings['flymenu_social_icon']))
                        $sffm_settings[$menu]['flymenu']['social_icon'] = $menu_type_settings['flymenu_social_icon'];
                    if (isset($menu_type_settings['flymenu_footer_custom_content']))
                        $sffm_settings[$menu]['flymenu']['footer_custom_content'] = wp_kses_post($menu_type_settings['flymenu_footer_custom_content']);
                    if (isset($menu_type_settings['flymenu_hide_screen']))
                        $sffm_settings[$menu]['flymenu']['hide_screen'] = $menu_type_settings['flymenu_hide_screen'];
                    if (isset($menu_type_settings['flymenu_display_condition'])) {
                        $sffm_settings[$menu]['flymenu']['specific_pages'] = isset($menu_type_settings['flymenu_specific_pages']) ? $menu_type_settings['flymenu_specific_pages'] : [];
                        $sffm_settings[$menu]['flymenu']['specific_archive'] = isset($menu_type_settings['flymenu_specific_archive']) ? $menu_type_settings['flymenu_specific_archive'] : [];
                        $sffm_settings[$menu]['flymenu']['cpt_pages'] = isset($menu_type_settings['flymenu_cpt_pages']) ? $menu_type_settings['flymenu_cpt_pages'] : [];
                    }

                    if (isset($menu_type_settings['flymenu_polylang_display_condition'])) {
                        $sffm_settings[$menu]['flymenu']['polylang_display_condition'] = isset($menu_type_settings['flymenu_polylang_display_condition']) ? $menu_type_settings['flymenu_polylang_display_condition'] : [];
                        $sffm_settings[$menu]['flymenu']['polylang_show_hide_languages'] = isset($menu_type_settings['flymenu_polylang_show_hide_languages']) ? $menu_type_settings['flymenu_polylang_show_hide_languages'] : [];
                    }

                    // Toggle Button Settings
                    if (isset($menu_type_settings['flymenu_toggle_button_icon_type']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['icon_type'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_icon_type']);
                    if (isset($menu_type_settings['flymenu_toggle_button_shape']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['shape'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_shape']);
                    if (isset($menu_type_settings['flymenu_toggle_button_hamburger_icon_type']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['hamburger_icon_type'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_hamburger_icon_type']);
                    if (isset($menu_type_settings['flymenu_toggle_button_open_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['open_icon'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_open_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_close_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['close_icon'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_close_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_custom_open_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['custom_open_icon'] = sanitize_url($menu_type_settings['flymenu_toggle_button_custom_open_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_custom_close_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['custom_close_icon'] = sanitize_url($menu_type_settings['flymenu_toggle_button_custom_close_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_hvr_animation']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['hvr_animation'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_hvr_animation']);
                    if (isset($menu_type_settings['flymenu_toggle_button_idle_animation']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['idle_animation'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_idle_animation']);
                    if (isset($menu_type_settings['flymenu_toggle_button_insert_on_menu'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['insert_on_menu'] = $menu_type_settings['flymenu_toggle_button_insert_on_menu'] == 'on' ? 'on' : 'off';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_menu']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['menu'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_menu']);
                    if (isset($menu_type_settings['flymenu_toggle_button_menu_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['menu_icon'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_menu_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_menu_icon_position']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['menu_icon_position'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_menu_icon_position']);
                    if (isset($menu_type_settings['flymenu_toggle_button_open_panel_on']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['open_panel_on'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_open_panel_on']);
                    if (isset($menu_type_settings['flymenu_toggle_button_position']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['position'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_position']);
                    if (isset($menu_type_settings['flymenu_toggle_button_offset_top'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['offset_top'] = is_numeric($menu_type_settings['flymenu_toggle_button_offset_top']) ? intval($menu_type_settings['flymenu_toggle_button_offset_top']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_offset_bottom'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['offset_bottom'] = is_numeric($menu_type_settings['flymenu_toggle_button_offset_bottom']) ? intval($menu_type_settings['flymenu_toggle_button_offset_bottom']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_offset_left'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['offset_left'] = is_numeric($menu_type_settings['flymenu_toggle_button_offset_left']) ? intval($menu_type_settings['flymenu_toggle_button_offset_left']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_offset_right'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['offset_right'] = is_numeric($menu_type_settings['flymenu_toggle_button_offset_right']) ? intval($menu_type_settings['flymenu_toggle_button_offset_right']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_size'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['size'] = is_numeric($menu_type_settings['flymenu_toggle_button_size']) ? intval($menu_type_settings['flymenu_toggle_button_size']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_icon_size'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['icon_size'] = is_numeric($menu_type_settings['flymenu_toggle_button_icon_size']) ? intval($menu_type_settings['flymenu_toggle_button_icon_size']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_image_size'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['image_size'] = is_numeric($menu_type_settings['flymenu_toggle_button_image_size']) ? intval($menu_type_settings['flymenu_toggle_button_image_size']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_hamburger_width'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['hamburger_width'] = is_numeric($menu_type_settings['flymenu_toggle_button_hamburger_width']) ? intval($menu_type_settings['flymenu_toggle_button_hamburger_width']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_hamburger_spacing'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['hamburger_spacing'] = is_numeric($menu_type_settings['flymenu_toggle_button_hamburger_spacing']) ? intval($menu_type_settings['flymenu_toggle_button_hamburger_spacing']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_hamburger_thickness'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['hamburger_thickness'] = is_numeric($menu_type_settings['flymenu_toggle_button_hamburger_thickness']) ? intval($menu_type_settings['flymenu_toggle_button_hamburger_thickness']) : '';
                    }

                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_icon_type']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_icon_type'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_iconmenu_icon_type']);
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_icon_type']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_hamburger_icon_type'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_icon_type']);
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_open_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_open_icon'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_iconmenu_open_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_close_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_close_icon'] = sanitize_text_field($menu_type_settings['flymenu_toggle_button_iconmenu_close_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_custom_open_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_custom_open_icon'] = sanitize_url($menu_type_settings['flymenu_toggle_button_iconmenu_custom_open_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_custom_close_icon']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_custom_close_icon'] = sanitize_url($menu_type_settings['flymenu_toggle_button_iconmenu_custom_close_icon']);
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_icon_size'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_icon_size'] = is_numeric($menu_type_settings['flymenu_toggle_button_iconmenu_icon_size']) ? intval($menu_type_settings['flymenu_toggle_button_iconmenu_icon_size']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_image_size'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_image_size'] = is_numeric($menu_type_settings['flymenu_toggle_button_iconmenu_image_size']) ? intval($menu_type_settings['flymenu_toggle_button_iconmenu_image_size']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_width'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_hamburger_width'] = is_numeric($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_width']) ? intval($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_width']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_spacing'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_hamburger_spacing'] = is_numeric($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_spacing']) ? intval($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_spacing']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_thickness'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['iconmenu_hamburger_thickness'] = is_numeric($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_thickness']) ? intval($menu_type_settings['flymenu_toggle_button_iconmenu_hamburger_thickness']) : '';
                    }

                    if (isset($menu_type_settings['flymenu_toggle_button_bg_color']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_bg_color'] = sffm_sanitize_color($menu_type_settings['flymenu_toggle_button_bg_color']);

                    if (isset($menu_type_settings['flymenu_toggle_button_hover_bg_color']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_hover_bg_color'] = sffm_sanitize_color($menu_type_settings['flymenu_toggle_button_hover_bg_color']);

                    if (isset($menu_type_settings['flymenu_toggle_button_icon_color']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_icon_color'] = sanitize_hex_color($menu_type_settings['flymenu_toggle_button_icon_color']);

                    if (isset($menu_type_settings['flymenu_toggle_button_hover_icon_color']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_hover_icon_color'] = sanitize_hex_color($menu_type_settings['flymenu_toggle_button_hover_icon_color']);

                    if (isset($menu_type_settings['flymenu_toggle_button_shadow_x'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_shadow_x'] = is_numeric($menu_type_settings['flymenu_toggle_button_shadow_x']) ? intval($menu_type_settings['flymenu_toggle_button_shadow_x']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_shadow_y'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_shadow_y'] = is_numeric($menu_type_settings['flymenu_toggle_button_shadow_y']) ? intval($menu_type_settings['flymenu_toggle_button_shadow_y']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_shadow_blur'])) {
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_shadow_blur'] = is_numeric($menu_type_settings['flymenu_toggle_button_shadow_blur']) ? intval($menu_type_settings['flymenu_toggle_button_shadow_blur']) : '';
                    }
                    if (isset($menu_type_settings['flymenu_toggle_button_shadow_color']))
                        $sffm_settings[$menu]['flymenu']['toggle_button']['button_shadow_color'] = sffm_sanitize_color($menu_type_settings['flymenu_toggle_button_shadow_color']);

                    if (isset($menu_type_settings['flymenu_layout']))
                        $sffm_settings[$menu]['flymenu']['flymenu_layout'] = sanitize_text_field($menu_type_settings['flymenu_layout']);

                    /* Text Fields */
                    $flymenu_text = array(
                        'custom_display_content_from',
                        'content_alignment',
                        'header_text_option',
                        'custom_header_text',
                        'custom_tagline_text',
                        'header_img_option',
                        'header_image_width_unit',
                        'searchform_area',
                        'searchform_placeholder',
                        'social_icons_area',
                        'social_icons_shape',
                        'header_content_alignment',
                        'footer_content_alignment',
                        'menu_animation',
                        'body_content_alignment',
                        'side_menu_submenu_animation',
                        'skew_side_menu_submenu_animation',
                        'skew_side_menu_inclination',
                        'icon_menu_submenu_animation',
                        'fullscreen_menu_column',
                        'fullscreen_menu_submenu_animation',
                        'fullscreen_content_width_unit',
                        'menu_hover_animation',
                        'overlay_type',
                        'display_condition',
                        'panel_position',
                        'panel_animation',
                        'fullscreen_panel_animation',
                        'panel_show_animation',
                        'panel_hide_animation',
                        'panel_width_unit',
                        'panel_background_type',
                        'panel_background_repeat',
                        'panel_background_size',
                        'panel_background_position',
                        'panel_background_attachment',
                        'submenu_toggle_open_icon',
                        'submenu_toggle_close_icon',
                        'close_button_position'
                    );
                    foreach ($flymenu_text as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key]))
                            $sffm_settings[$menu]['flymenu'][$key] = sanitize_text_field($menu_type_settings['flymenu_' . $key]);
                    }

                    /* URL Fields */
                    $flymenu_url = array(
                        'custom_header_image',
                        'panel_background_image',
                        'panel_video_url',
                        'panel_video_background_image',
                    );
                    foreach ($flymenu_url as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key]))
                            $sffm_settings[$menu]['flymenu'][$key] = sanitize_url($menu_type_settings['flymenu_' . $key]);
                    }

                    /* Typography */
                    $flymenu_typos = array(
                        'header_title',
                        'header_tagline',
                        'header_text',
                        'menu_label',
                        'menu_desc',
                        'menu_content',
                        'footer_text'
                    );

                    foreach ($flymenu_typos as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key . '_font_family']))
                            $sffm_settings[$menu]['flymenu']['' . $key . '_font_family'] = sanitize_text_field($menu_type_settings['flymenu_' . $key . '_font_family']);
                        if (isset($menu_type_settings['flymenu_' . $key . '_font_style']))
                            $sffm_settings[$menu]['flymenu']['' . $key . '_font_style'] = sanitize_text_field($menu_type_settings['flymenu_' . $key . '_font_style']);
                        if (isset($menu_type_settings['flymenu_' . $key . '_text_transform']))
                            $sffm_settings[$menu]['flymenu']['' . $key . '_text_transform'] = sanitize_text_field($menu_type_settings['flymenu_' . $key . '_text_transform']);
                        if (isset($menu_type_settings['flymenu_' . $key . '_text_decoration']))
                            $sffm_settings[$menu]['flymenu']['' . $key . '_text_decoration'] = sanitize_text_field($menu_type_settings['flymenu_' . $key . '_text_decoration']);
                        if (isset($menu_type_settings['flymenu_' . $key . '_font_size'])) {
                            $sffm_settings[$menu]['flymenu']['' . $key . '_font_size'] = is_numeric($menu_type_settings['flymenu_' . $key . '_font_size']) ? intval($menu_type_settings['flymenu_' . $key . '_font_size']) : '';
                        }
                        if (isset($menu_type_settings['flymenu_' . $key . '_line_height'])) {
                            $sffm_settings[$menu]['flymenu']['' . $key . '_line_height'] = is_numeric($menu_type_settings['flymenu_' . $key . '_line_height']) ? floatval($menu_type_settings['flymenu_' . $key . '_line_height']) : '';
                        }
                        if (isset($menu_type_settings['flymenu_' . $key . '_letter_spacing'])) {
                            $sffm_settings[$menu]['flymenu']['' . $key . '_letter_spacing'] = is_numeric($menu_type_settings['flymenu_' . $key . '_letter_spacing']) ? floatval($menu_type_settings['flymenu_' . $key . '_letter_spacing']) : '';
                        }
                    }

                    /* Spacing */
                    $flymenu_spacing = array(
                        'panel_spacing',
                        'header_spacing',
                        'menu_spacing',
                        'footer_spacing',
                        'menu_item_spacing',
                        'search_spacing',
                        'close_button_offset'
                    );
                    $flymenu_spacing_positions = array('top', 'right', 'bottom', 'left');
                    foreach ($flymenu_spacing as $key) {
                        foreach ($flymenu_spacing_positions as $position) {
                            if (isset($menu_type_settings['flymenu_' . $key][$position])) {
                                $sffm_settings[$menu]['flymenu'][$key][$position] = is_numeric($menu_type_settings['flymenu_' . $key][$position]) ? intval($menu_type_settings['flymenu_' . $key][$position]) : '';
                            }
                        }
                    }

                    /* Color */
                    $flymenu_colors = array(
                        'header_title_color',
                        'header_tagline_color',
                        'header_text_color',
                        'menu_label_color',
                        'menu_bg_color',
                        'menu_border_color',
                        'menu_label_color_hover',
                        'menu_bg_color_hover',
                        'menu_border_color_hover',
                        'menu_short_desc_color',
                        'menu_add_content_color',
                        'menu_notification_color',
                        'menu_notification_bg_color',
                        'footer_text_color',
                        'search_bg_color',
                        'search_border_color',
                        'search_text_color',
                        'search_button_color',
                        'search_button_icon_color',
                        'socialicon_icon_color',
                        'socialicon_bg_color',
                        'socialicon_border_color',
                        'socialicon_tooltip_color',
                        'socialicon_tooltip_bg_color',
                        'socialicon_icon_color_hover',
                        'socialicon_bg_color_hover',
                        'socialicon_border_color_hover',
                        'close_button_icon_color',
                        'close_button_icon_color_hover',
                        'close_button_bg_color',
                        'close_button_bg_color_hover',
                        'panel_wave_color_1',
                        'panel_wave_color_2',
                        'panel_wave_color_3',
                        'panel_wave_color_4',
                        'panel_background_color'
                    );
                    foreach ($flymenu_colors as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key]))
                            $sffm_settings[$menu]['flymenu'][$key] = sanitize_hex_color($menu_type_settings['flymenu_' . $key]);
                    }

                    /* Alpha Color */
                    $flymenu_alphacolors = array(
                        'overlay_color',
                        'scrollbar_drag_rail_color',
                        'scrollbar_drag_bar_color',
                        'panel_overlay_color',
                        'panel_video_overlay_color',
                    );
                    foreach ($flymenu_alphacolors as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key]))
                            $sffm_settings[$menu]['flymenu'][$key] = sffm_sanitize_color($menu_type_settings['flymenu_' . $key]);
                    }

                    /* Number */
                    $flymenu_numbers = array(
                        'header_image_width',
                        'header_element_spacing',
                        'search_form_width',
                        'search_form_border_radius',
                        'footer_element_spacing',
                        'skew_side_menu_angle',
                        'fullscreen_content_width',
                        'panel_width',
                        'scrollbar_width',
                        'zindex',
                        'social_icons_button_size',
                        'social_icons_icon_size',
                        'close_button_size',
                        'close_button_border_radius'
                    );
                    foreach ($flymenu_numbers as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key]))
                            $sffm_settings[$menu]['flymenu'][$key] = is_numeric($menu_type_settings['flymenu_' . $key]) ? intval($menu_type_settings['flymenu_' . $key]) : '';
                    }

                    /* CheckBox */
                    $flymenu_checkbox = array(
                        'parent_menu_clickable',
                        'auto_close_panel',
                        'social_link_new_tab',
                        'freeze_scrollbar',
                        'front_pages',
                        'blog_pages',
                        'archive_pages',
                        'error_pages',
                        'search_pages',
                        'panel_show_scrollbar',
                    );

                    foreach ($flymenu_checkbox as $key) {
                        if (isset($menu_type_settings['flymenu_' . $key]))
                            $sffm_settings[$menu]['flymenu'][$key] = $menu_type_settings['flymenu_' . $key] == 'on' ? 'on' : 'off';
                    }
                }

                $default_sffm_settings[$menu] = SFFM_Admin_Templates::default_flyfloat_settings();
                $sffm_settings = $this->recursive_parse_args($sffm_settings, $default_sffm_settings);
                $updated = $sffm_settings[$menu];

                update_option('sffm_settings', $sffm_settings);
                wp_send_json_success($updated);
            }
        } else {
            die('No script kiddies please!');
        }
        die();
    }

    /**
     * Ajax function to save menu item data
     */
    public function save_menu_item_data() {

        if (isset($_POST['wp_nonce']) && wp_verify_nonce($_POST['wp_nonce'], 'sffm-backend-ajax-nonce')) {
            parse_str(stripslashes_deep($_POST['data']), $data);
            $updated = $data;

            // Save menu item data
            foreach ($data['menu-item-mega'] as $id => $meta) {
                $old_meta = get_post_meta($id, 'sffm_item_settings', true);
                $old_meta = $this->recursive_parse_args($old_meta, SFFM_Admin_Templates::default_settings());
                $meta = $this->recursive_parse_args($meta, $old_meta);
                $updated['menu-item-mega'][$id] = $meta;
                update_post_meta($id, 'sffm_item_settings', $meta);
            }
            wp_send_json_success($updated);
        }
        die();
    }

    public function sffm_menu_select() {

        if (isset($_POST['wp_nonce']) && wp_verify_nonce($_POST['wp_nonce'], 'sffm-backend-ajax-nonce')) {
            if (!empty($_POST['menu_id'])) {
                $sffm_selected_menuid = sanitize_text_field($_POST['menu_id']);

                $sffm_settings = is_array(get_option('sffm_settings')) ? get_option('sffm_settings') : array();
                $sffm_settings['menu_' . $sffm_selected_menuid]['menu_enabled'] = true;
                $sffm_settings['menu_' . $sffm_selected_menuid]['menu_type'] = isset($sffm_settings['menu_' . $sffm_selected_menuid]['menu_type']) ? $sffm_settings['menu_' . $sffm_selected_menuid]['menu_type'] : 'floating_menu';
                $sffm_settings['menu_' . $sffm_selected_menuid]['menu_id'] = sanitize_text_field($_POST['menu_id']);

                update_option('sffm_settings', $sffm_settings);

                die();
            }
        }
        wp_die();
    }

    public function sffm_menu_deselect() {

        if (isset($_POST['wp_nonce']) && wp_verify_nonce($_POST['wp_nonce'], 'sffm-backend-ajax-nonce')) {
            if (!empty($_POST['menu_id'])) {
                $sffm_selected_menuid = sanitize_text_field($_POST['menu_id']);

                $sffm_settings = get_option('sffm_settings');
                $sffm_settings['menu_' . $sffm_selected_menuid]['menu_enabled'] = false;

                update_option('sffm_settings', $sffm_settings);

                die();
            }
        }
        wp_die();
    }

    public function sffm_menu_floatfly() {

        if (isset($_POST['wp_nonce']) && wp_verify_nonce($_POST['wp_nonce'], 'sffm-backend-ajax-nonce')) {
            if (!empty($_POST['menu_id'])) {
                $sffm_selected_menuid = sanitize_text_field($_POST['menu_id']);

                $sffm_settings = get_option('sffm_settings');
                $sffm_settings['menu_' . $sffm_selected_menuid]['menu_type'] = sanitize_text_field($_POST['menu_type']);
                $sffm_settings['menu_' . $sffm_selected_menuid]['menu_id'] = sanitize_text_field($_POST['menu_id']);

                update_option('sffm_settings', $sffm_settings);

                die();
            }
        }
        wp_die();
    }

    /**
     * Register the stylesheets for the admin area.
     *
     */
    public function enqueue_styles() {

        wp_enqueue_media();
        /* Select2 */
        wp_enqueue_style('jquery-select2', SFFM_URL . 'admin/css/select2.min.css', array(), $this->version);

        wp_enqueue_style('sffm-menuicons', SFFM_URL . 'admin/css/icons/menuicons.css', array(), $this->version);
        wp_enqueue_style('sffm-closeicons', SFFM_URL . 'admin/css/icons/closeicons.css', array(), $this->version);
        wp_enqueue_style('fontawesome-6.3.0', SFFM_URL . 'admin/css/icons/fontawesome-6.3.0.css', array(), $this->version);
        wp_enqueue_style('essentialicon', SFFM_URL . 'admin/css/icons/essentialicon.css', array(), $this->version);
        wp_enqueue_style('sffm-eleganticons', SFFM_URL . 'admin/css/icons/eleganticons.css', array(), $this->version);
        wp_enqueue_style('iconfont', SFFM_URL . 'admin/css/icons/icofont.css', array(), $this->version);
        wp_enqueue_style('materialdesignicons', SFFM_URL . 'admin/css/icons/materialdesignicons.css', array(), $this->version);

        /* Enqueue jQuery Chosen */
        wp_enqueue_style('chosen', SFFM_URL . 'admin/css/chosen.css', array(), $this->version);

        /* For color picker */
        wp_enqueue_style('wp-color-picker');

        if (is_rtl()) {
            wp_enqueue_style($this->plugin_name, SFFM_URL . '/admin/css/sffm-admin.rtl.css');
        } else {
            wp_enqueue_style($this->plugin_name, SFFM_URL . '/admin/css/sffm-admin.css');
        }
    }

    /**
     * Register the JavaScript for the admin area.
     *
     */
    public function enqueue_scripts($hook) {
        wp_register_script('jquery-select2', SFFM_URL . 'admin/js/select2.min.js', array('jquery'), $this->version, true);
        if ('nav-menus.php' == $hook) {
            /* Enqueue jQuery Chosen */
            wp_enqueue_script('chosen-script', SFFM_URL . 'admin/js/chosen.jquery.js', array('jquery'), $this->version);

            /* For color picker */
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_script('wp-color-picker-alpha-min', SFFM_URL . 'admin/js/wp-color-picker-alpha.min.js', array('wp-color-picker'), $this->version);

            wp_enqueue_script('jquery-condition', SFFM_URL . 'admin/js/jquery-condition.js', array('jquery'), $this->version);

            wp_enqueue_script($this->plugin_name, SFFM_URL . 'admin/js/super-floating-flying-menu-admin.js', array('jquery', 'jquery-ui-sortable', 'jquery-select2'), $this->version);
        }

        if ('toplevel_page_super-floating-flying-menu' == $hook) {
            wp_enqueue_script('sffm-floatfly-menu-options', SFFM_URL . 'admin/js/sffm-options.js', array('jquery'), $this->version);
            wp_localize_script('sffm-floatfly-menu-options', 'sffmOptionsData', array(
                'ajax_url' => admin_url('admin-ajax.php'),
                'admin_nonce' => wp_create_nonce('sffm-backend-ajax-nonce'),
            ));
        }
    }

    /**
     * Register nav menus page metabox with mega menu settings.
     *
     * @return void
     */
    public function register_nav_meta_box() {

        global $pagenow;

        if ('nav-menus.php' !== $pagenow) {
            return;
        }

        add_meta_box(
            'super-floating-flying-menu-settings', esc_html__('Super Floating & Flying Menu', 'super-floating-flying-menu'), array($this, 'render_metabox'), 'nav-menus', 'side', 'high'
        );
    }

    /**
     * Render nav menus metabox
     *
     * @return void
     */
    public function render_metabox() {
        include plugin_dir_path(__FILE__) . 'inc/settings-nav.php';
    }

    public function get_selected_menu_id() {

        if (null !== $this->current_menu_id) {
            return $this->current_menu_id;
        }

        $nav_menus = wp_get_nav_menus(array('orderby' => 'name'));
        $menu_count = count($nav_menus);
        $nav_menu_selected_id = isset($_REQUEST['menu']) ? (int) $_REQUEST['menu'] : 0;
        $add_new_screen = (isset($_GET['menu']) && 0 == $_GET['menu']) ? true : false;

        $this->current_menu_id = $nav_menu_selected_id;

        // If we have one theme location, and zero menus, we take them right into editing their first menu
        $page_count = wp_count_posts('page');
        $one_theme_location_no_menus = (1 == count(get_registered_nav_menus()) && !$add_new_screen && empty($nav_menus) && !empty($page_count->publish)) ? true : false;

        // Get recently edited nav menu
        $recently_edited = absint(get_user_option('nav_menu_recently_edited'));
        if (empty($recently_edited) && is_nav_menu($this->current_menu_id)) {
            $recently_edited = $this->current_menu_id;
        }

        // Use $recently_edited if none are selected
        if (empty($this->current_menu_id) && !isset($_GET['menu']) && is_nav_menu($recently_edited)) {
            $this->current_menu_id = $recently_edited;
        }

        // On deletion of menu, if another menu exists, show it
        if (!$add_new_screen && 0 < $menu_count && isset($_GET['action']) && 'delete' == $_GET['action']) {
            $this->current_menu_id = $nav_menus[0]->term_id;
        }

        // Set $this->current_menu_id to 0 if no menus
        if ($one_theme_location_no_menus) {
            $this->current_menu_id = 0;
        } elseif (empty($this->current_menu_id) && !empty($nav_menus) && !$add_new_screen) {
            // if we have no selection yet, and we have menus, set to the first one in the list
            $this->current_menu_id = $nav_menus[0]->term_id;
        }

        return $this->current_menu_id;
    }

    /**
     * Get settings from DB
     *
     */
    public function get_settings($menu_id) {
        return get_term_meta($menu_id, $this->menu_item_meta_key, true);
    }

    protected function recursive_parse_args($args, $defaults) {

        $new_args = (array) $defaults;

        foreach ($args as $key => $value) {
            if (is_array($value) && isset($new_args[$key])) {
                $new_args[$key] = $this->recursive_parse_args($value, $new_args[$key]);
            } else {
                $new_args[$key] = $value;
            }
        }

        return $new_args;
    }

    public function update_menu_template() {
        $templates = get_option('sffm_menu_template');
        if (!$templates) {
            $templates = array();
            $menus = get_terms('nav_menu', array('hide_empty' => false, 'parent' => 0));
            foreach ($menus as $menu) {
                $menu_id = $menu->term_id;
                $templ_id = get_post_meta($menu_id, 'sffm-menu-template', true);
                delete_post_meta($menu_id, 'sffm-menu-template', $templ_id);
                if ($templ_id && 'publish' == get_post_status($templ_id)) {
                    $templates[$menu->term_id] = $templ_id;
                }
            }
            update_option('sffm_menu_template', $templates);
        }
    }

    public function delete_nav_menu_action($term_id) {
        $templates = get_option('sffm_menu_template');
        $elemntor_temp_id = isset($templates[$term_id]) ? $templates[$term_id] : null;
        unset($templates[$term_id]);
        if (!is_null(get_post($elemntor_temp_id))) {
            wp_delete_post($elemntor_temp_id, true);
        }
        update_option('sffm_menu_template', $templates);
    }

    public function add_plugin_action_link($links) {
        $custom = array(
            'settings' => sprintf('<a href="%s" aria-label="%s">%s</a>', esc_url(add_query_arg('page', 'super-floating-flying-menu', admin_url('admin.php'))), esc_attr__('SFFM Settings', 'super-floating-flying-menu'), esc_attr__('Settings', 'super-floating-flying-menu')),
            'config' => sprintf('<a href="%s" aria-label="%s">%s</a>', esc_url(admin_url('nav-menus.php')), esc_attr__('Configure', 'super-floating-flying-menu'), esc_attr__('Config', 'super-floating-flying-menu'))
        );

        return array_merge($custom, (array) $links);
    }

    public function update_sffm_settings($term_id) {
        $sffm_settings = get_option('sffm_settings');
        $term_id = 'menu_' . $term_id;
        if (isset($sffm_settings[$term_id])) {
            unset($sffm_settings[$term_id]);
            update_option('sffm_settings', $sffm_settings);
        }
    }

    public function register_translation_strings() {
        $sffm_settings = get_option('sffm_settings');

        if (isset($sffm_settings) && !empty($sffm_settings) && is_array($sffm_settings)) {

            foreach ($sffm_settings as $mkey => $sffm_setting) {
                $menu_id = $sffm_setting['menu_id'];
                if (!Super_Floating_Flying_Menu_Public::check_menu_exists($menu_id)) {
                    continue;
                }

                $flymenu_settings = isset($sffm_setting['flymenu']) ? $sffm_setting['flymenu'] : null;
                if ($flymenu_settings) {
                    $string_array = array(
                        'Menu Custom Header Text' => isset($flymenu_settings['custom_header_text']) ? $flymenu_settings['custom_header_text'] : '',
                        'Menu Custom Tagline Text' => isset($flymenu_settings['custom_tagline_text']) ? $flymenu_settings['custom_tagline_text'] : '',
                        'Menu Header Custom Content' => isset($flymenu_settings['header_custom_content']) ? $flymenu_settings['header_custom_content'] : '',
                        'Menu Footer Custom Content' => isset($flymenu_settings['footer_custom_content']) ? $flymenu_settings['footer_custom_content'] : '',
                        'Menu Search Form Placeholder' => isset($flymenu_settings['searchform_placeholder']) ? $flymenu_settings['searchform_placeholder'] : '',
                    );
                    foreach ($string_array as $title => $strings) {
                        if (has_action('wpml_register_single_string')) {
                            do_action('wpml_register_single_string', 'Super Floating and Flying Menu', $menu_id . ' - ' . $title, $strings);
                        }
                    }
                }

                $sffm_menu_items = wp_get_nav_menu_items($menu_id);
                foreach ($sffm_menu_items as $item) {
                    $item_id = $item->ID;
                    $sffm_item_settings = get_post_meta($item_id, 'sffm_item_settings', true);
                    $string_array = array(
                        'Menu Item Tooltip Text' => isset($sffm_item_settings['floatingmenu']['tooltip_text']) ? $sffm_item_settings['floatingmenu']['tooltip_text'] : '',
                        'Menu Item Short Description' => isset($sffm_item_settings['flymenu']['short_description']) ? $sffm_item_settings['flymenu']['short_description'] : '',
                        'Menu Item Additional Content' => isset($sffm_item_settings['flymenu']['additional_content']) ? $sffm_item_settings['flymenu']['additional_content'] : '',
                        'Push Menu Item Additional Content' => isset($sffm_item_settings['flymenu']['push_menu_additional_content']) ? $sffm_item_settings['flymenu']['push_menu_additional_content'] : '',
                        'Menu Item Notification Label' => isset($sffm_item_settings['flymenu']['notification_label']) ? $sffm_item_settings['flymenu']['notification_label'] : '',
                    );

                    foreach ($string_array as $title => $strings) {
                        if (has_action('wpml_register_single_string')) {
                            do_action('wpml_register_single_string', 'Super Floating and Flying Menu', $item_id . ' - ' . $title, $strings);
                        }
                    }
                }
            }
        }
    }

}
