<?php

namespace SFFM;

if (!defined('ABSPATH'))
    exit();

class SFFM_Widget_Loader {

    private static $instance = null;

    /**
     * Initialize integration hooks
     *
     * @return void
     */
    public function __construct() {
        spl_autoload_register([$this, 'autoload']);

        $this->includes();
        // Elementor hooks
        $this->add_actions();
    }

    function add_elementor_widget_categories() {

        $groups = array(
            'super-floating-flying-menu' => esc_html__('SFFM Elements', 'super-floating-flying-menu')
        );

        foreach ($groups as $key => $value) {
            \Elementor\Plugin::$instance->elements_manager->add_category($key, ['title' => $value], 1);
        }
    }

    /**
     * we loaded module manager + admin php from here
     * @return [type] [description]
     */
    private function includes() {
        require SFFM_PATH . 'admin/inc/elements/includes/module-manager.php';
    }

    /**
     * Autoload Classes
     *
     * @since 1.6.0
     */
    public function autoload($class) {
        if (0 !== strpos($class, __NAMESPACE__)) {
            return;
        }

        $has_class_alias = isset($this->classes_aliases[$class]);

        // Backward Compatibility: Save old class name for set an alias after the new class is loaded
        if ($has_class_alias) {
            $class_alias_name = $this->classes_aliases[$class];
            $class_to_load = $class_alias_name;
        } else {
            $class_to_load = $class;
        }

        if (!class_exists($class_to_load)) {

            $filename = strtolower(
                preg_replace(
                    ['/^' . __NAMESPACE__ . '\\\/', '/([a-z])([A-Z])/', '/_/', '/\\\/'], ['', '$1-$2', '-', DIRECTORY_SEPARATOR], $class_to_load
                )
            );

            $filename = SFFM_PATH . 'admin/inc/elements/' . $filename . '.php';
            if (is_readable($filename)) {
                include($filename);
            }
        }

        if ($has_class_alias) {
            class_alias($class_alias_name, $class);
        }
    }

    /**
     * Add Actions
     *
     * @since 0.1.0
     *
     * @access private
     */
    public function add_actions() {
        add_action('elementor/init', [$this, 'add_elementor_widget_categories']);

        // Fires after Elementor controls are registered.
        add_action('elementor/controls/controls_registered', [$this, 'register_controls']);

        //FrontEnd Scripts
        add_action('elementor/frontend/before_register_scripts', [$this, 'register_frontend_scripts']);
        add_action('elementor/frontend/after_enqueue_scripts', [$this, 'enqueue_frontend_scripts']);

        //FrontEnd Styles
        add_action('elementor/frontend/before_register_styles', [$this, 'register_frontend_styles']);
        add_action('elementor/frontend/after_enqueue_styles', [$this, 'enqueue_frontend_styles']);

        //Editor Scripts
        add_action('elementor/editor/before_enqueue_scripts', [$this, 'enqueue_editor_scripts']);

        //Editor Style
        add_action('elementor/editor/after_enqueue_styles', [$this, 'enqueue_editor_styles']);

        //Fires after Elementor preview styles are enqueued.
        add_action('elementor/preview/enqueue_styles', [$this, 'enqueue_preview_styles']);
    }

    /**
     * Register Controls
     * @since 1.0.0
     * @access public
     */
    public function register_controls() {

    }

    /**
     * Register Frontend Scripts
     * @since 1.0.0
     * @access public
     */
    public function register_frontend_scripts() {

    }

    /**
     * Enqueue Frontend Scripts
     * @since 1.0.0
     * @access public
     */
    public function enqueue_frontend_scripts() {
        wp_enqueue_script('sffm-charming', SFFM_URL . 'admin/inc/elements/assets/js/charming.min.js', array('jquery'), SFFM_VERSION, true);
        wp_enqueue_script('sffm-anime', SFFM_URL . 'admin/inc/elements/assets/js/anime.min.js', array('jquery'), SFFM_VERSION, true);
        wp_enqueue_script('sffm-hover-menu', SFFM_URL . 'admin/inc/elements/assets/js/hover-menu.js', array('jquery'), SFFM_VERSION, true);
        wp_enqueue_script('sffm-multi-level-menu', SFFM_URL . 'admin/inc/elements/assets/js/multi-level-menu.js', array('jquery'), SFFM_VERSION, true);
        wp_enqueue_script('sffm-el-frontend', SFFM_URL . 'admin/inc/elements/assets/js/frontend.js', array('jquery'), SFFM_VERSION, true);
    }

    /**
     * Register Frontend Styles
     * @since 1.0.0
     * @access public
     */
    public function register_frontend_styles() {

    }

    /**
     * Enqueue Frontend Styles
     * @since 1.0.0
     * @access public
     */
    public function enqueue_frontend_styles() {
        if (is_rtl()) {
            wp_enqueue_style('sffm-frontend', SFFM_URL . 'admin/inc/elements/assets/css/frontend.rtl.css', array(), SFFM_VERSION);
        } else {
            wp_enqueue_style('sffm-frontend', SFFM_URL . 'admin/inc/elements/assets/css/frontend.css', array(), SFFM_VERSION);
        }
    }

    /**
     * Enqueue Editor Scripts
     * @since 1.0.0
     * @access public
     */
    public function enqueue_editor_scripts() {
        wp_enqueue_script('sffm-elements-editor', SFFM_URL . 'admin/inc/elements/assets/js/editor.js', array('jquery'), SFFM_VERSION, true);
        wp_localize_script('sffm-elements-editor', 'is_elementor_pro_installed', array($this, 'is_elementor_pro_installed'));
    }

    /**
     * Enqueue Editor Styles
     * @since 1.0.0
     * @access public
     */
    public function enqueue_editor_styles() {
        wp_enqueue_style('sffm-editor-style', SFFM_URL . 'admin/inc/elements/assets/css/editor-styles.css', array(), SFFM_VERSION);
    }

    /**
     * Preview Styles
     * @since 1.0.0
     * @access public
     */
    public function enqueue_preview_styles() {

    }

    /**
     * Check if theme has elementor Pro installed
     *
     * @return boolean
     */
    public function is_elementor_pro_installed() {
        return function_exists('elementor_pro_load_plugin') ? 'yes' : 'no';
    }

    /**
     * Creates and returns an instance of the class
     * @since 1.0.0
     * @access public
     * return object
     */
    public static function get_instance() {
        if (self::$instance == null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

}

if (!function_exists('sffm_widget_loader')) {

    /**
     * Returns an instance of the plugin class.
     * @since  1.0.0
     * @return object
     */
    function sffm_widget_loader() {
        return SFFM_Widget_Loader::get_instance();
    }

}
sffm_widget_loader();
